@@ -32,12 +32,16 @@
 #include "util/XMLHelper.h"
 #include "util/XMLConstants.h"
 
+#include <strstream>
 #include <boost/lambda/bind.hpp>
 #include <boost/lambda/if.hpp>
 #include <boost/lambda/lambda.hpp>
+
 #include <xercesc/framework/MemBufFormatTarget.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
+#include <zlib.h>
 
+using namespace xmltooling::logging;
 using namespace xmltooling;
 using namespace xercesc;
 using namespace boost::lambda;
@@ -53,9 +57,9 @@ bool XMLHelper::hasXSIType(const DOMElement* e)
 
 xmltooling::QName* XMLHelper::getXSIType(const DOMElement* e)
 {
-    DOMAttr* attribute = e->getAttributeNodeNS(xmlconstants::XSI_NS, type);
+    DOMAttr* attribute = e ? e->getAttributeNodeNS(xmlconstants::XSI_NS, type) : nullptr;
     if (attribute) {
-        const XMLCh* attributeValue = attribute->getTextContent();
+        const XMLCh* attributeValue = attribute->getNodeValue();
         if (attributeValue && *attributeValue) {
             int i;
             if ((i=XMLString::indexOf(attributeValue,chColon))>0) {
@@ -77,7 +81,7 @@ xmltooling::QName* XMLHelper::getXSIType(const DOMElement* e)
 
 DOMAttr* XMLHelper::getIdAttribute(const DOMElement* domElement)
 {
-    if(!domElement->hasAttributes()) {
+    if(!domElement || !domElement->hasAttributes()) {
         return nullptr;
     }
     
@@ -176,17 +180,24 @@ xmltooling::QName* XMLHelper::getNodeQName(const DOMNode* domNode)
     return nullptr; 
 }
 
-xmltooling::QName* XMLHelper::getAttributeValueAsQName(const DOMAttr* attribute)
-{
-    return getNodeValueAsQName(attribute);
-}
-
 xmltooling::QName* XMLHelper::getNodeValueAsQName(const DOMNode* domNode)
 {
     if (!domNode)
         return nullptr;
     
-    const XMLCh* value=domNode->getTextContent();
+    const XMLCh* value = nullptr;
+    XMLCh* ownedValue = nullptr;
+    
+    if (domNode->getNodeType() == DOMNode::ATTRIBUTE_NODE) {
+        value = domNode->getNodeValue();
+    }
+    else if (domNode->getNodeType() == DOMNode::ELEMENT_NODE) {
+        ownedValue = getWholeTextContent(static_cast<const DOMElement*>(domNode));
+        value = ownedValue;
+    }
+
+    ArrayJanitor<XMLCh> jan(ownedValue);
+
     if (!value || !*value)
         return nullptr;
 
@@ -195,22 +206,39 @@ xmltooling::QName* XMLHelper::getNodeValueAsQName(const DOMNode* domNode)
         XMLCh* prefix=new XMLCh[i+1];
         XMLString::subString(prefix,value,0,i);
         prefix[i]=chNull;
-        xmltooling::QName* ret=new xmltooling::QName(domNode->lookupNamespaceURI(prefix), value + i + 1, prefix);
-        delete[] prefix;
-        return ret;
+        ArrayJanitor<XMLCh> jan2(prefix);
+        const XMLCh* ns = domNode->lookupNamespaceURI(prefix);
+        if (!ns) {
+            auto_ptr_char temp(prefix);
+            throw XMLToolingException("Namespace prefix ($1) not declared in document.", params(1, temp.get()));
+        }
+        return new xmltooling::QName(ns, value + i + 1, prefix);
     }
     
     return new xmltooling::QName(domNode->lookupNamespaceURI(nullptr), value);
 }
 
-bool XMLHelper::getNodeValueAsBool(const xercesc::DOMNode* domNode, bool def)
+bool XMLHelper::getNodeValueAsBool(const DOMNode* domNode, bool def)
 {
     if (!domNode)
         return def;
-    const XMLCh* value = domNode->getNodeValue();
+
+    const XMLCh* value = nullptr;
+    XMLCh* ownedValue = nullptr;
+
+    if (domNode->getNodeType() == DOMNode::ATTRIBUTE_NODE) {
+        value = domNode->getNodeValue();
+    }
+    else if (domNode->getNodeType() == DOMNode::ELEMENT_NODE) {
+        ownedValue = getWholeTextContent(static_cast<const DOMElement*>(domNode));
+        value = ownedValue;
+    }
+
+    ArrayJanitor<XMLCh> jan(ownedValue);
+
     if (!value || !*value)
         return def;
-    if (*value == chLatin_t || *value == chDigit_1)
+    else if (*value == chLatin_t || *value == chDigit_1)
         return true;
     else if (*value == chLatin_f || *value == chDigit_0)
         return false;
@@ -228,16 +256,46 @@ DOMElement* XMLHelper::appendChildElement(DOMElement* parentElement, DOMElement*
     return childElement;
 }
 
-bool XMLHelper::isNodeNamed(const xercesc::DOMNode* n, const XMLCh* ns, const XMLCh* local)
+bool XMLHelper::isNodeNamed(const DOMNode* n, const XMLCh* ns, const XMLCh* local)
 {
     return (n && XMLString::equals(local,n->getLocalName()) && XMLString::equals(ns,n->getNamespaceURI()));
 }
 
+XMLCh* XMLHelper::getWholeTextContent(const DOMElement* e)
+{
+    XMLCh* buf = nullptr;
+    const DOMNode* child = e ? e->getFirstChild() : nullptr;
+    while (child) {
+        if (child->getNodeType() == DOMNode::TEXT_NODE || child->getNodeType() == DOMNode::CDATA_SECTION_NODE) {
+            if (child->getNodeValue()) {
+                if (buf) {
+                    XMLSize_t initialLen = buf ? XMLString::stringLen(buf) : 0;
+                    XMLCh* merged = new XMLCh[initialLen + XMLString::stringLen(child->getNodeValue()) + 1];
+                    XMLString::copyString(merged, buf);
+                    XMLString::catString(merged + initialLen, child->getNodeValue());
+                    delete[] buf;
+                    buf = merged;
+                }
+                else {
+                    buf = new XMLCh[XMLString::stringLen(child->getNodeValue()) + 1];
+                    XMLString::copyString(buf, child->getNodeValue());
+                }
+            }
+        }
+        else if (child->getNodeType() != DOMNode::COMMENT_NODE) {
+            break;
+        }
+        child = child->getNextSibling();
+    }
+
+    return buf;
+}
+
 const XMLCh* XMLHelper::getTextContent(const DOMElement* e)
 {
     DOMNode* child = e ? e->getFirstChild() : nullptr;
     while (child) {
-        if (child->getNodeType() == DOMNode::TEXT_NODE)
+        if (child->getNodeType() == DOMNode::TEXT_NODE || child->getNodeType() == DOMNode::CDATA_SECTION_NODE)
             return child->getNodeValue();
         child = child->getNextSibling();
     }
@@ -372,7 +430,7 @@ bool XMLHelper::getCaseSensitive(const xercesc::DOMElement* e, bool defValue, co
     if (e) {
         const XMLCh* ic = e->getAttributeNS(ns, ignoreCase);
         if (ic && * ic) {
-            logging::Category::getInstance(XMLTOOLING_LOGCAT ".XMLHelper").warn("Deprecated attribute \"ignoreCase\" encountered in configuration. Use \"caseSensitive\".");
+            Category::getInstance(XMLTOOLING_LOGCAT ".XMLHelper").warn("DEPRECATED: attribute \"ignoreCase\" encountered in configuration. Use \"caseSensitive\".");
 
             // caseInsensitive = !"ignoreCase"
             if (*ic == chLatin_t || *ic == chDigit_1)
@@ -383,7 +441,7 @@ bool XMLHelper::getCaseSensitive(const xercesc::DOMElement* e, bool defValue, co
         const XMLCh* ci = e->getAttributeNS(ns, caseSensitive);
         if (ci && *ci) {
             if (ic && *ic) {
-                logging::Category::getInstance(XMLTOOLING_LOGCAT ".XMLHelper").warn("Attribute \"ignoreCase\" and \"caseSensitive\" should not be used in the same element.");
+                Category::getInstance(XMLTOOLING_LOGCAT ".XMLHelper").warn("Attribute \"ignoreCase\" and \"caseSensitive\" should not be used in the same element.");
             }
             if (*ci == chLatin_t || *ci == chDigit_1) {
                 result =  true;
@@ -396,6 +454,37 @@ bool XMLHelper::getCaseSensitive(const xercesc::DOMElement* e, bool defValue, co
     return result;
 }
 
+void XMLHelper::encode(std::ostream& os, const char* str)
+{
+    size_t pos;
+    while (str && *str) {
+        pos = strcspn(str, "\"<>&");
+        if (pos > 0) {
+            os.write(str, pos);
+            str += pos;
+        }
+        else {
+            switch (*str) {
+            case '"':   os << "&quot;";     break;
+            case '<':   os << "&lt;";       break;
+            case '>':   os << "&gt;";       break;
+            case '&':   os << "&amp;";      break;
+            default:    os << *str;
+            }
+            str++;
+        }
+    }
+}
+
+std::string XMLHelper::encode(const char* str)
+{
+    ostrstream stream;
+
+    encode(stream, str);
+    stream << ends;
+    return std::string(stream.str());
+}
+
 void XMLHelper::serialize(const DOMNode* n, std::string& buf, bool pretty)
 {
     static const XMLCh impltype[] = { chLatin_L, chLatin_S, chNull };
@@ -404,7 +493,6 @@ void XMLHelper::serialize(const DOMNode* n, std::string& buf, bool pretty)
     MemBufFormatTarget target;
     DOMImplementation* impl=DOMImplementationRegistry::getDOMImplementation(impltype);
 
-#ifdef XMLTOOLING_XERCESC_COMPLIANT_DOMLS
     DOMLSSerializer* serializer = static_cast<DOMImplementationLS*>(impl)->createLSSerializer();
     XercesJanitor<DOMLSSerializer> janitor(serializer);
     if (pretty && serializer->getDomConfig()->canSetParameter(XMLUni::fgDOMWRTFormatPrettyPrint, pretty))
@@ -415,15 +503,6 @@ void XMLHelper::serialize(const DOMNode* n, std::string& buf, bool pretty)
     theOutput->setByteStream(&target);
     if (!serializer->write(n, theOutput))
         throw XMLParserException("unable to serialize XML");
-#else
-    DOMWriter* serializer = static_cast<DOMImplementationLS*>(impl)->createDOMWriter();
-    XercesJanitor<DOMWriter> janitor(serializer);
-    serializer->setEncoding(UTF8);
-    if (pretty && serializer->canSetFeature(XMLUni::fgDOMWRTFormatPrettyPrint, pretty))
-        serializer->setFeature(XMLUni::fgDOMWRTFormatPrettyPrint, pretty);
-    if (!serializer->writeNode(&target, *n))
-        throw XMLParserException("unable to serialize XML");
-#endif
 
     buf.erase();
     buf.append(reinterpret_cast<const char*>(target.getRawBuffer()),target.getLen());
@@ -436,7 +515,7 @@ namespace {
         StreamFormatTarget(std::ostream& out) : m_out(out) {}
         ~StreamFormatTarget() {}
 
-        void writeChars(const XMLByte *const toWrite, const xsecsize_t count, XMLFormatter *const formatter) {
+        void writeChars(const XMLByte *const toWrite, const XMLSize_t count, XMLFormatter *const formatter) {
             m_out.write(reinterpret_cast<const char*>(toWrite),count);
         }
 
@@ -457,7 +536,6 @@ ostream& XMLHelper::serialize(const DOMNode* n, ostream& out, bool pretty)
     StreamFormatTarget target(out);
     DOMImplementation* impl=DOMImplementationRegistry::getDOMImplementation(impltype);
 
-#ifdef XMLTOOLING_XERCESC_COMPLIANT_DOMLS
     DOMLSSerializer* serializer = static_cast<DOMImplementationLS*>(impl)->createLSSerializer();
     XercesJanitor<DOMLSSerializer> janitor(serializer);
     if (pretty && serializer->getDomConfig()->canSetParameter(XMLUni::fgDOMWRTFormatPrettyPrint, pretty))
@@ -468,15 +546,6 @@ ostream& XMLHelper::serialize(const DOMNode* n, ostream& out, bool pretty)
     theOutput->setByteStream(&target);
     if (!serializer->write(n, theOutput))
         throw XMLParserException("unable to serialize XML");
-#else
-    DOMWriter* serializer=(static_cast<DOMImplementationLS*>(impl))->createDOMWriter();
-    XercesJanitor<DOMWriter> janitor(serializer);
-    serializer->setEncoding(UTF8);
-    if (pretty && serializer->canSetFeature(XMLUni::fgDOMWRTFormatPrettyPrint, pretty))
-        serializer->setFeature(XMLUni::fgDOMWRTFormatPrettyPrint, pretty);
-    if (!serializer->writeNode(&target,*n))
-        throw XMLParserException("unable to serialize XML");
-#endif
 
     return out;
 }
@@ -496,3 +565,112 @@ ostream& xmltooling::operator<<(ostream& ostr, const XMLObject& obj)
         throw XMLParserException(msg.get());
     }
 }
+
+namespace {
+    extern "C" {
+        voidpf saml_zalloc(void* opaque, uInt items, uInt size)
+        {
+            return malloc(items*size);
+        }
+
+        void saml_zfree(void* opaque, voidpf addr)
+        {
+            free(addr);
+        }
+    };
+};
+
+char* XMLHelper::deflate(char* in, unsigned int in_len, unsigned int* out_len)
+{
+    z_stream z;
+    memset(&z, 0, sizeof(z_stream));
+
+    z.zalloc = saml_zalloc;
+    z.zfree = saml_zfree;
+    z.opaque = nullptr;
+    z.next_in = (Bytef*)in;
+    z.avail_in = in_len;
+    *out_len = 0;
+
+    int ret = deflateInit2(&z, 9, Z_DEFLATED, -15, 9, Z_DEFAULT_STRATEGY);
+    if (ret != Z_OK) {
+        Category::getInstance(XMLTOOLING_LOGCAT ".XMLHelper").error("zlib deflateInit2 failed with error code (%d)", ret);
+        return nullptr;
+    }
+
+    int dlen = in_len + (in_len >> 8) + 12;  /* orig_size * 1.001 + 12 */
+    char* out = new char[dlen];
+    z.next_out = (Bytef*)out;
+    z.avail_out = dlen;
+
+    ret = ::deflate(&z, Z_FINISH);
+    if (ret != Z_STREAM_END) {
+        deflateEnd(&z);
+        Category::getInstance(XMLTOOLING_LOGCAT ".XMLHelper").error("zlib deflateInit2 failed with error code (%d)", ret);
+        delete[] out;
+    }
+
+    *out_len = z.total_out;
+    deflateEnd(&z);
+    return out;
+}
+
+unsigned int XMLHelper::inflate(char* in, unsigned int in_len, ostream& out)
+{
+    z_stream z;
+    memset(&z, 0, sizeof(z_stream));
+
+    z.zalloc = saml_zalloc;
+    z.zfree = saml_zfree;
+    z.opaque = nullptr;
+    z.next_in = (Bytef*)in;
+    z.avail_in = in_len;
+
+    int dlen = in_len << 3;  /* guess inflated size: orig_size * 8 */
+    Byte* buf = new Byte[dlen];
+    memset(buf, 0, dlen);
+    z.next_out = buf;
+    z.avail_out = dlen;
+
+    int ret = inflateInit2(&z, -15);
+    if (ret != Z_OK) {
+        Category::getInstance(XMLTOOLING_LOGCAT ".XMLHelper").error("zlib inflateInit2 failed with error code (%d)", ret);
+        delete[] buf;
+        return 0;
+    }
+
+    size_t diff;
+    int iter = 30;
+    while (--iter) {  /* Make sure we can never be caught in infinite loop */
+        ret = ::inflate(&z, Z_SYNC_FLUSH);
+        switch (ret) {
+        case Z_STREAM_END:
+            diff = z.next_out - buf;
+            z.next_out = buf;
+            while (diff--)
+                out << *(z.next_out++);
+            goto done;
+
+        case Z_OK:  /* avail_out should be 0 now. Time to dump the buffer. */
+            diff = z.next_out - buf;
+            z.next_out = buf;
+            while (diff--)
+                out << *(z.next_out++);
+            memset(buf, 0, dlen);
+            z.next_out = buf;
+            z.avail_out = dlen;
+            break;
+
+        default:
+            delete[] buf;
+            inflateEnd(&z);
+            Category::getInstance(XMLTOOLING_LOGCAT ".XMLHelper").error("zlib inflate failed with error code (%d)", ret);
+            return 0;
+        }
+    }
+done:
+    delete[] buf;
+    int out_len = z.total_out;
+    inflateEnd(&z);
+    return out_len;
+}
